#!/usr/bin/perl
# Name: Web Quake 3 Arena Management CGI
# Description: Perl CGI interface to administer quake 3 servers and players on them
# Version: 0.6
# Authors: Jason Jorgensen <jasonj@innominatus.com>
# Copyright (C) 2003  Jason Jorgensen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###########################################################################

use CGI qw/:standard :html3 :Carp/;
#use diagnostics;
use strict;
use Socket;
use IO::Socket;
#use Getopt::Long;
use Sys::Hostname;
use BSU::DynamicTemplate;
use CGI::Session::DB_File;
use DBI;
use DBIx::Broker;

my $programname = "Quake 3 Server Web Admin";

my $debug = 0;
my $showbanned = 1;
my $timeout = 1;
my $pausetime = 0;
my $script = $ENV{'SCRIPT_NAME'};
my $serverstemplate = 'templates/servers.html';
my $rconservertemplate = 'templates/rconserver.html';
my $servertemplate = 'templates/server.html';
my $aliastemplate = 'templates/aliases.html';
my $serversconfig = 'q3servers.cgi';
my $serversconfigdelimiter = '::';
my $navigation; # variable used to call different screens.

my $dbuser = 'q3admin';
my $dbpassword = '<DATABASEPASSWORD>';
my $dbdatabase = 'Q3Admin';
my $dbhost = 'localhost';
my $dbtype = 'mysql';
my $dbport = '3306';
my $dbtable = 'Aliases';

#my $socket;

## If you want q3admin to record names and ip's set $q3record to 1
my $aliasdb = eval {DBIx::Broker->new($dbtype, $dbdatabase, $dbhost, $dbport, $dbuser, $dbpassword)};
if ($debug) { $aliasdb->debug_on; }

my $startform = start_form();
chomp $startform;

my @servers;

&read_config(); ## Read configuration file that contains all the quake3 servers

## Retrieve session id either from CGI variables, a cookie, or make it undefined
my $sessionid = param("webq3sid") || cookie('webq3sid') || undef;

## Regardless of the sessionid, start up a new session
my $session = new CGI::Session::DB_File($sessionid, {FileName=>'/tmp/webq3admin_sessions.db', LockDirectory=>'/tmp'});
if (! $session->param('navigation')) { $session = undef; $session = new CGI::Session::DB_File(undef, {FileName=>'/tmp/webq3admin_sessions.db', LockDirectory=>'/tmp'}); }

## Pass a cookie to the users browser with the current session id        
my $cookie = cookie(-name=>'webq3sid', -value=>$session->id, -expires=>"+3d");
                        
print header(-cookie=>$cookie);
if (! $aliasdb) { print "Can not connect with $programname database"; exit; }
print "DEBUG: sessionid: $sessionid<br>\n" if $debug;

## Actions
my $action = param("action") || "";

if ($action eq 'ban') { 
  my $number = param("number");
  my $name = param("name");
  my $ip = param("ip");
  my $counter = param("counter") || $session->param('counter');
  
  my $server = $servers[$counter-1];
  print "BAN/KICKING NUMBER $number $name $ip<br>\n";
  print "Server Response:<br>";
  my @array = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} g_filterBan 1");
  foreach my $line (@{$array[0]}) { print $line."<br>\n"; }
  sleep 1;
  @array = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} addip $ip");
  foreach my $line (@{$array[0]}) { print $line."<br>\n"; }
  sleep 1;
  @array = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} clientkick $number");
  foreach my $line (@{$array[0]}) { print $line."<br>\n"; }
  print "<hr><br>";

#  &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} g_filterBan 1; $server->{'rconpassword'} addip $ip; $server->{'rconpassword'} clientkick $number;");
  sleep 1;
#  &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} g_filterBan 1;\n");
#  sleep 1;
#  &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} addip $ip;\n");
#  sleep 1;
#  &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} clientkick $number;\n");
#  sleep 1;
}

if ($action eq 'kick') { 
  my $number = param("number");
  my $name = param("name");
  my $counter = param("counter") || $session->param('counter');
  
  my $server = $servers[$counter-1];
  print "KICKING NUMBER $number $name<br>\n";
  print "Server Response:<br>";
  my @array = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} clientkick $number");
  foreach my $line (@{$array[0]}) { print $line."<br>\n"; }
  print "<hr><br>";
  sleep 1;
}

if ($action eq 'unban') { 
  my $banip = param("banip");
  my $counter = param("counter") || $session->param('counter');
  
  my $server = $servers[$counter-1];
  print "UNBANING IP $banip<br>\n";
  print "Server Response:<br>";
  my @array = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} removeip $banip");
  foreach my $line (@{$array[0]}) { print $line."<br>\n"; }
  print "<hr><br>";
  sleep 1;
}

if ($action eq 'command') { 
  my $command = param("command");
  my $counter = param("counter") || $session->param('counter');
  
  my $server = $servers[$counter-1];
  print "Sending Command: $command<br>\n";
  print "Server Response:<br>";
  my @array = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} $command");
  foreach my $line (@{$array[0]}) { print $line."<br>\n"; }
  print "<hr><br>";
  sleep 1;
}

if ($action eq 'maprestart') { 
  my $counter = param("counter") || $session->param('counter');
  
  my $server = $servers[$counter-1];
  print "RESTARTING SERVER $server->{'hostname'} <br><br><br>\n";
  print "Server Response:<br>";
  my @array = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} map_restart\n");
  foreach my $line (@{$array[0]}) { print $line."<br>\n"; }
  print "<hr><br>";
  sleep 1;
}


## Navigation/Screens
print "DEBUG: param navigation: ".param("navigation")."<br>\n" if $debug;
$navigation = param("navigation") || $session->param('navigation') || 'servers';
print "DEBUG: session navigation: ".$session->param('navigation')."<br>\n" if $debug;
print "DEBUG: navigation: $navigation<br>\n" if $debug;

if ($navigation eq 'server') { $session->param(-name=>'navigation', -value=>$navigation); &display_server(); }
elsif ($navigation eq 'servers') { $session->param(-name=>'navigation', -value=>$navigation); &display_servers(); }
elsif ($navigation eq 'aliases') { &display_aliases(); }
else { &display_servers(); }

## Close out session so that DBFile is written
print "DEBUG: navigation: $navigation<br>\n" if $debug;
print "DEBUG: closing session<br>\n" if $debug;
$session->close;

$aliasdb->disconnect();

exit;


## Sub Routines
sub display_aliases {
  my @aliases;
  my $ip = param("ip");
  my $name = param("name");
  $name =~ s/\'/\\'/g;
  my $playerfont = param("playerfont") || $session->param('playerfont') || 'Normal'; # user selection to calculate $refresh

  print "DEBUG: ip: $ip<BR>\n" if $debug;
  print "DEBUG: name: $name<BR>\n" if $debug;

  my $aliasentry;
  my @aliasdata;
  my @fields  =  ( "Name", "IP" );

  if ($ip && $name) {
    @aliases = $aliasdb->select(\@fields, $dbtable, 'WHERE Name=\''.$name.'\' AND IP=\''.$ip.'\'', 1);    
  }
  elsif ($ip) {
    @aliases = $aliasdb->select(\@fields, $dbtable, "WHERE IP='$ip'", 1);    
  }
  elsif ($name) {
    @aliases = $aliasdb->select(\@fields, $dbtable, "WHERE Name='$name'", 1);    
  }
  else {
    return;
  }

  foreach my $alias (@aliases) {
    $aliasentry = {
                    name => &prettyname($alias->{'Name'}, $playerfont),
                    ip   => $alias->{'IP'},
                  };    
    push @aliasdata, $aliasentry;
  }
  my $aliastemplate = new BSU::DynamicTemplate "$aliastemplate";
  $aliastemplate->set_loop_variables(aliastable => \@aliasdata);
  print $aliastemplate->render();  
}

sub display_server {
  my @playersdata;
  my $counter = param("counter") || $session->param('counter');
  my $autorefresh = param("autorefresh") || $session->param('autorefresh') || 'Never'; # user selection to calculate $refresh
  my $refresh;  # used to set the auto refresh meta tag
  my $playerfont = param("playerfont") || $session->param('playerfont') || 'Normal'; # user selection to calculate $refresh
  my $fontsize;
  
  my $server = $servers[$counter-1];
  #my $socket = &q3open($server->{'host'}, $server->{'port'}, $timeout);
  my @temparray;
  my $response;
  my $game;
  my $playerdata;

  ## Get the current info for just this one selected server
  my $serverdata;
  sleep $pausetime;
  $response = &q3command($server->{'host'}, $server->{'port'}, $timeout, "getinfo");
  my @serverarray = split /\\/, @$response[0];
  splice (@serverarray,0,1);

  ## Get the current list of players for just this one selected server
  sleep $pausetime;
  if ($server->{'rconpassword'}) {
    $response = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} status");
  }
  else {
    $response = &q3command($server->{'host'}, $server->{'port'}, $timeout, "getstatus");
  }
  if (! $response) { &problemdown(); }
  my @players = @$response;
  if ($server->{'rconpassword'}) {
    splice (@players,0,3);
  }
  else {
    splice (@players,0,1);
  }
  foreach my $player (@players) {
    if ($player =~ /\S+/) {
      my $playerdata;
      my $ip;
      my $port;

      my @splitplayer = split /\s+/, $player;
      my @playername;
      my $playername;
      if ($server->{'rconpassword'}) {
        splice (@splitplayer,0,1);
        @playername = splice (@splitplayer,3,$#splitplayer-6,'UNUSED');
        $playername = join ' ', @playername;
        ($ip, $port) = split /:/, $splitplayer[5];
        $playerdata = {
                     number  => $splitplayer[0],
                     score   => $splitplayer[1],
                     ping    => $splitplayer[2],
                     name    => $playername,
                     lastmsg => $splitplayer[4],
                     ip      => $ip,
                     port    => $port,
                     qport   => $splitplayer[6],
                     rate    => $splitplayer[7],
                    };    
       }
       else {
         @playername = splice (@splitplayer,2,$#splitplayer,'UNUSED');
         $playername = join ' ', @playername;
         $playerdata = {
                     score   => $splitplayer[0],
                     ping    => $splitplayer[1],
                     name    => $playername,
                    };    
      }
      print "DEBUG: player line: $player<br>\n" if $debug;
      print "DEBUG: Playername: @playername<br>\n" if $debug;
      print "DEBUG: splitplayer[0]: $splitplayer[0]<br>\n" if $debug;
      print "DEBUG: splitplayer[1]: $splitplayer[1]<br>\n" if $debug;
      print "DEBUG: splitplayer[2]: $splitplayer[2]<br>\n" if $debug;
      print "DEBUG: splitplayer[3]: $splitplayer[3]<br>\n" if $debug;
      print "DEBUG: splitplayer[4]: $splitplayer[4]<br>\n" if $debug;
      print "DEBUG: splitplayer[5]: $splitplayer[5]<br>\n" if $debug;
      print "DEBUG: splitplayer[6]: $splitplayer[6]<br>\n" if $debug;
      print "DEBUG: splitplayer[7]: $splitplayer[7]<br>\n" if $debug;

      my $correctedname = $playerdata->{'name'};
      print "DEBUG: name: $correctedname<br>\n" if $debug;
      $correctedname =~ s/\"//g;
      $correctedname =~ s/\^/\^/g;
      $correctedname =~ s/\$/\$/g;
      print "DEBUG: stripped name: $correctedname<br>\n" if $debug;
      &record($correctedname,$playerdata->{'ip'});
      #if ($q3record) { &record($playerdata->{'name'},$playerdata->{'ip'}); }

      ## convert player name into images
      $playerdata->{'name'} = &prettyname($playerdata->{'name'}, $playerfont);


      if ($server->{'rconpassword'}) {
        $playerdata->{'ipaliaslink'} = "$script?navigation=aliases&ip=$ip";
        $playerdata->{'kickbutton'} = $startform.submit(-name=>'Kick').hidden(-name=>'action',-override=>1,-value=>'kick').hidden(-name=>'number',-override=>1,-value=>$playerdata->{'number'}).end_form;
        $playerdata->{'banbutton'} = $startform.submit(-name=>'Ban').hidden(-name=>'action',-override=>1,-value=>'ban').hidden(-name=>'number',-override=>1,-value=>$playerdata->{'number'}).hidden(-name=>'ip',-override=>1,-value=>$playerdata->{'ip'}).end_form;
      }
      $playerdata->{'namealiaslink'} = "$script?navigation=aliases&name=$correctedname";
      #$playerdata->{'ipaliasbutton'} = $startform.submit(-name=>'Aliases').hidden(-name=>'ip', -override=>1,-value=>$ip).hidden(-name=>'navigation',-override=>1,-value=>'aliases').end_form;
      #$playerdata->{'namealiasbutton'} = $startform.submit(-name=>'IPs').hidden(-name=>'name', -override=>1,-value=>$correctedname).hidden(-name=>'navigation',-override=>1,-value=>'aliases').end_form;

      print "$player<br>\n" if $debug;
      push @playersdata, $playerdata;
    }
  }
  


  ## Get the current variables for this one selected server
  my @variablesdata;
    sleep $pausetime;
  $response = &q3command($server->{'host'}, $server->{'port'}, $timeout, "getstatus");
  if (! $response) { &problemdown(); }
  my @variables = split /\\/, @$response[0];
  splice (@variables,0,1);
  for (my $c = 0; $c < $#variables; $c = $c+2) {
    my $variabledata;
    $variabledata->{'name'} = $variables[$c];
    $variabledata->{'value'} = $variables[($c+1)];
    $variabledata->{'field'} = textfield(-name=>var-$variables[$c],
                                          -default=>"",
                                          -override=>1,
                                          -size=>(length($variables[($c+1)]) + 1),
                                          -maxlength=>(length($variables[($c+1)]) * 2));
    
    print "$variables[$c] => $variables[($c+1)]<br>\n" if $debug;
    push @variablesdata, $variabledata;
  }

  
  ## Get currently banned IP's
  my @bannedipsdata;
  if (($showbanned) && ($server->{'rconpassword'})){
    sleep $pausetime;
    my @bannediparray = &q3command($server->{'host'}, $server->{'port'}, $timeout, "rcon $server->{'rconpassword'} g_banIps");
    my ($bannedips) = (${$bannediparray[0]}[0] =~ /\:\"(.*)\^7\"\ default/);
    my @bannedips = split / /, $bannedips || undef;
    foreach my $banip (@bannedips) { 
      my $bannedipdata;
      $bannedipdata->{'banip'} = $banip;
      $bannedipdata->{'unbanbutton'} = $startform.submit(-name=>'Unban').hidden(-name=>'action',-override=>1,-value=>'unban').hidden(-name=>'banip',-override=>1,-value=>$banip).end_form; 
      push @bannedipsdata, $bannedipdata;
    }
  }

  #&q3close($socket);
  #if (! $serverdata->{'game'}) { $serverdata->{'game'} = &gametype($serverdata->{'gametype'}); }
  
  ## Calculate the autorefresh time
  if ($autorefresh eq "Never") { $refresh = ""; }
  elsif ($autorefresh eq "15 Seconds") { $refresh = "<META HTTP-EQUIV=\"REFRESH\" content=\"15\">"; }
  elsif ($autorefresh eq "30 Seconds") { $refresh = "<META HTTP-EQUIV=\"REFRESH\" content=\"30\">"; }
  elsif ($autorefresh eq "1 Minute") { $refresh = "<META HTTP-EQUIV=\"REFRESH\" content=\"60\">"; }
  elsif ($autorefresh eq "5 Minutes") { $refresh = "<META HTTP-EQUIV=\"REFRESH\" content=\"300\">"; }
  elsif ($autorefresh eq "10 Minutes") { $refresh = "<META HTTP-EQUIV=\"REFRESH\" content=\"600\">"; }
  else { $refresh = ""; }

  
 my $memememe;
  if ($server->{'rconpassword'}) {
    $memememe = $rconservertemplate;
  }
  else {
    $memememe = $servertemplate;
  }
  my $servertemplate = new BSU::DynamicTemplate "$memememe";
  for (my $c = 0; $c < $#serverarray; $c = $c+2) {
    $servertemplate->set($serverarray[$c] => $serverarray[($c+1)]);
    if ($serverarray[$c] eq 'gametype' && ! grep (/^game$/, @serverarray)) { 
      $servertemplate->set(game =>  &gametype($serverarray[($c+1)])); 
      print "DEBUG: game type: $serverarray[($c+1)]<br>\n" if $debug;
      print "DEBUG: game type: $game<br>\n" if $debug;
    }
  }
  $servertemplate->set(
                        startform => $startform,
                        endform => end_form(),
                        serversbutton => $startform.submit(-name=>'All Servers Page').hidden(-name=>'navigation',-override=>1, -value=>'servers').end_form,
                        refreshbutton => $startform.submit(-name=>'Refresh Page').end_form,
                        autorefreshpopup => $startform.popup_menu(-name=> 'autorefresh', -values=>['Never','15 Seconds','30 Seconds','1 Minute','5 Minutes','10 Minutes'], -default=>"$autorefresh").submit(-name=>'Set Auto Refresh').end_form,
                        playerfont => $startform.popup_menu(-name=> 'playerfont', -values=>['Text','Normal','Big','Huge'], -default=>"$playerfont").submit(-name=>'Set Font Type').end_form,
                        serverhidden => hidden(-name=>'counter',-override=>1, -value=>$counter),
                        refresh => $refresh,
                        host => $server->{'host'},
                        port => $server->{'port'},
                       );
  if ($server->{'rconpassword'}) {
    $servertemplate->set(
                        restartbutton => $startform.submit(-name=>'Map Restart').hidden(-name=>'action',-override=>1, -value=>'maprestart').end_form,
                        commandfield => $startform.textfield(-name=>'command',
                                          -default=>"",
                                          -override=>1,
                                          -size=>'50',
                                          -maxlength=>'256').submit(-name=>'Send Command').hidden(-name=>'action',-override=>1, -value=>'command').end_form,
                      );
  }  
  $servertemplate->set_loop_variables(bannediptable => \@bannedipsdata);
  $servertemplate->set_loop_variables(playerstable => \@playersdata);
  $servertemplate->set_loop_variables(variablestable => \@variablesdata);

  ## Before we render and print the screen, store any useful variables in the session
  $session->param(-name=>'counter', -value=>$counter);
  $session->param(-name=>'autorefresh', -value=>$autorefresh);
  $session->param(-name=>'playerfont', -value=>$playerfont);

  ## Render and print the screen
  print $servertemplate->render();  
}

sub display_servers {
  ## if no navigation variables are populated display list of servers to be managed
  my @serversdata;

  foreach my $server (@servers) {
    #my $socket = &q3open($server->{'host'}, $server->{'port'}, $timeout);
    my $serverdata;
    my @temparray;
    my $response = &q3command($server->{'host'}, $server->{'port'}, $timeout, "getinfo");
    @temparray = split /\\/, @$response[0];
    splice (@temparray,0,1);
    for (my $c = 0; $c < $#temparray; $c = $c+2) {
      print "$temparray[$c] => $temparray[($c+1)]<br>\n" if $debug;
      $serverdata->{$temparray[$c]} = $temparray[($c+1)];
    }
    if ($server->{'rconpassword'}) {
      $serverdata->{'button'} = $startform.submit(-name=>'Manage').hidden(-name=>'counter', -override=>1,-value=>$server->{'counter'}).hidden(-name=>'navigation',-override=>1,-value=>'server').end_form;
    }
    else {
      $serverdata->{'button'} = $startform.submit(-name=>'View').hidden(-name=>'counter', -override=>1,-value=>$server->{'counter'}).hidden(-name=>'navigation',-override=>1,-value=>'server').end_form;
    }
    $serverdata->{'counter'} = $server->{'counter'};
    $serverdata->{'host'} = $server->{'host'};
    $serverdata->{'port'} = $server->{'port'};
    if (! $serverdata->{'game'}) { $serverdata->{'game'} = &gametype($serverdata->{'gametype'}); }
    push @serversdata, $serverdata;
    #&q3close($socket);
  }

  my $serverstemplate = new BSU::DynamicTemplate "$serverstemplate";
  $serverstemplate->set(
                        refreshbutton => $startform.submit(-name=>'Refresh Page').end_form,
                        startform => $startform,
                        endform => end_form(),
                       );
  $serverstemplate->set_loop_variables(serverstable => \@serversdata);
  print $serverstemplate->render();  
  
}

sub gametype {
  my $type = shift @_;
  
  if (! $type) { return "Unknown, Down, or Between Levels"; }

  if ($type == 0) { return "Free for All";}     # 0 - Free For All DM (FFA)  [default]
  if ($type == 1) { return "1v1 DeathMatch";}   # 1 - 1v1 DM
  if ($type == 2) { return "Single Player";}    # 2 - Single-Player
  if ($type == 3) { return "Team Deathmatch";}  # 3 - Team Deathmatch
  if ($type == 4) { return "Capture The Flag";} # 4 - Capture The Flag (CTF) Teamplay
  if ($type == 5) { return "Clan Arena";}       # 5 - Clan Arena (CA)

  return "Unknown";

}

sub prettyname {
  my $name = shift @_;
  my $font = shift @_;
  my $fontsize;

  $name =~ s/^\"//g;
  $name =~ s/\"$//g;
  if ($font eq "Text") {
    $name = "<font color=white>$name";
    $name =~ s/\^1/\<\/font\>\<font color\=red\>/g;
    $name =~ s/\^2/\<\/font\>\<font color\=green\>/g;
    $name =~ s/\^3/\<\/font\>\<font color\=yellow\>/g;
    $name =~ s/\^4/\<\/font\>\<font color\=blue\>/g;
    $name =~ s/\^5/\<\/font\>\<font color\=lightblue\>/g;
    $name =~ s/\^6/\<\/font\>\<font color\=violet\>/g;
    $name =~ s/\^7/\<\/font\>\<font color\=white\>/g;
    $name =~ s/\^8/\<\/font\>\<font color\=\#2F4F4F\>/g;
    $name .= "</font>";
  }
  else {
    if ($font eq "Normal") { $fontsize = ""; }
    elsif ($font eq "Big") { $fontsize = "-BIG"; }
    elsif ($font eq "Huge") { $fontsize = "-HUGE"; }
    else {$fontsize = ""; }
    my @name = split //, $name;
    $name = "";
    my $color = "white";
    for (my $c=0; $c <= $#name; $c++) {
      if ($name[$c] eq '^') {
        if ($name[$c+1] == 1) { $color = "red"; }
        if ($name[$c+1] == 2) { $color = "green"; }
        if ($name[$c+1] == 3) { $color = "yellow"; }
        if ($name[$c+1] == 4) { $color = "dblue"; }
        if ($name[$c+1] == 5) { $color = "lblue"; }
        if ($name[$c+1] == 6) { $color = "violet"; }
        if ($name[$c+1] == 7) { $color = "white"; }
        if ($name[$c+1] == 8) { $color = "black"; }
        $c++;
      }
      else { $name .= "<img src=\"dec-images/$color-".ord($name[$c])."$fontsize.png\">"; }
    }
  }
  return $name;
}


sub read_config {
  my $counter = 1;
  open CONFIG, "<$serversconfig";
  foreach my $line (<CONFIG>) {
    ## If the line in the config file matches our scheme then add it to the server hash
    if ($line =~ /^.*$serversconfigdelimiter\d*/) {
      (my $host, my $port, my $rconpassword) = split $serversconfigdelimiter, $line;
      chomp ($host, $port, $rconpassword);
      if (($host =~ /.+\.\w+/ || $host =~ /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/) && $port =~ /\d+/) {
      my $temphash = { counter      => $counter,
                       host         => $host,
                       port         => $port,
		       rconpassword => $rconpassword,
		     };
      push @servers, $temphash;
      $counter++;
      }
    }
  }
  close CONFIG;
  if ($debug) {
    foreach my $server (@servers) {
      print "counter: $server->{'counter'}\n";
      print "host: $server->{'host'}\n";
      print "port: $server->{'port'}\n";
      print "rconpassword: $server->{rconpassword}\n";
    }
  }
}


sub problemdown {
  print <<END;
The server you are querying is either:<br>\n
- Down<br>\n
- Changing Maps<br>\n
- Unreachable<br>\n
<br>\n
This is a potential error received from the server:<br>
<PRE>
@_
</PRE>
<br>\n
<CENTER>Refresh this page in a few seconds to see if the server is responding yet</CENTER><br>\n
</BODY>
</HTML>
END

exit 0;
}

sub record {
  my $name = shift @_;
  $name =~ s/\'/\\'/g;
  my $ip = shift @_;

  if (($ip) && ($name)) {
    if (! $aliasdb->count($dbtable, 'WHERE Name=\''.$name.'\' AND IP=\''.$ip.'\''))  {
      my %alias = (
                   "Name", $name,
                   "IP", $ip,
                  );
      $aliasdb->insert($dbtable, \%alias);
    }
  }
}

sub q3open {
  my $host = shift @_;
  my $port = shift @_;
  my $timeout = shift @_;

  my $socket = IO::Socket::INET->new(PeerAddr => $host,
                                     PeerPort => $port,
                                     Proto    => 'udp',
                                     Timeout  => $timeout,
                                    );
#                                     Reuse    => 1,
#                                     Listen   => 10,
  if (! $socket) {
   print "Cant talk to server at '$host:$port'\n";
   die "Cant talk to server at '$host:$port'\n";
   exit;
  }
  $socket->autoflush(1);
  return $socket;
}

sub q3close {
  my $socket = shift @_;

  close $socket;
}


sub q3command {
#$debug=0;
  my $host = shift @_ || "";
  my $port = shift @_ || "";
  my $timeout = shift @_ || "";
  my $msg = shift @_ || "";
  my $buffer;
  my $code = "\377\377\377\377";
  $msg = $code.$msg;
 
  my $ok=1;
  my $sock=IO::Socket::INET->new(PeerAddr => $host,
                            PeerPort => $port,
                            Proto    => 'udp')
                or $ok=0;
  if ($ok)  {
  print $sock $msg;
  eval {
    local $SIG{ALRM} = sub { die "alarm\n" }; # NB: \n required
    alarm $timeout;
    #$sock->sysread($buffer,65507);  
    $sock->sysread($buffer,1024);
    alarm 0;
  };
  if ($@) {
    # die unless $@ eq "alarm\n";   # propagate unexpected errors
    # timed out
    #print "ERRORS: $@\n";
  }
  else {
    # didn't
    #print "DEBUG: $buffer\n";
  }       
  close($sock);
  #my @response=split(/\\/,$buffer);
  my@response = split /^/m, $buffer;
  shift @response;
  
  return \@response;
  }
}
